"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.coordsToCounter = coordsToCounter;
exports.resolveCounter = resolveCounter;
exports.resolveRowJump = resolveRowJump;
exports.getSquareDimensions = getSquareDimensions;
exports.jumpRow = jumpRow;
var emptyValue = {
  item: undefined,
  coords: undefined
};

function counterToCoords(counter, namedGroups) {
  var groups = toGroups(namedGroups);
  counter = normalizeCounter(counter, groups);

  if (counter == null) {
    return;
  }

  for (var i = 0; i < groups.length; i++) {
    var size = groups[i].length;

    if (counter < size) {
      return [i, counter];
    }

    counter = counter - size;
  }
}
/**
 * Converted 2d coordinates in a flat 1d integer referred everywhere as counter.
 * @param {Array[number, number]} coords The 2d coordinates to find an emoji
 * @param {Array<{name: string, emojis: string[]}>} namedGroups
 * @returns
 */


function coordsToCounter(coords, namedGroups) {
  var groups = toGroups(namedGroups);
  var counter = 0;

  for (var i = 0; i < coords[0]; i++) {
    counter += groups[i].length;
  }

  counter += coords[1];
  return counter;
}

function resolveCounter(counter, namedGroups) {
  var coords = counterToCoords(counter, namedGroups);

  if (!coords) {
    return emptyValue;
  }

  var groups = toGroups(namedGroups);
  return {
    item: groups[coords[0]][coords[1]],
    coords: coords
  };
}
/**
 * Helps calculate the position resulting from a jump between rows
 *
 * @param {*} counter
 * @param {1, -1} direction up or down
 * @param {*} jump the positive integer value to offset the counter by
 * @param {*} namedGroups
 * @returns a new counter
 */


function resolveRowJump(counter) {
  var direction = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
  var jump = arguments.length > 2 ? arguments[2] : undefined;
  var namedGroups = arguments.length > 3 ? arguments[3] : undefined;
  var {
    coords
  } = jumpRow(counter, direction, jump, namedGroups);

  if (coords == null) {
    return null;
  }

  return coordsToCounter(coords, namedGroups);
}

function getNextGroup(groupIndex, groups) {
  var direction = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
  var newIndex = groupIndex + direction;

  if (groups.length === 0) {
    return;
  }

  while (newIndex < 0) {
    newIndex += groups.length;
  }

  newIndex = newIndex % groups.length;
  return newIndex;
}

function getSquareDimensions(_ref) {
  var {
    rowWidth,
    squareMargin,
    squareSide
  } = _ref;
  var squareFullWidth = squareSide + 2 * squareMargin; // -2 to account for borders and safety

  var rowCount = Math.floor((rowWidth - 2) / squareFullWidth);
  var containerWidth = rowCount * squareFullWidth;
  return {
    containerWidth,
    rowCount
  };
}

function jumpRow(counter) {
  var direction = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
  var jump = arguments.length > 2 ? arguments[2] : undefined;
  var namedGroups = arguments.length > 3 ? arguments[3] : undefined;
  var coords = counterToCoords(counter, namedGroups);

  if (!coords) {
    return emptyValue;
  }

  var groups = toGroups(namedGroups);
  var groupIndex = coords[0];
  var itemIndex = coords[1];
  var groupSize = groups[groupIndex].length;
  var newIndex = direction === 1 ? itemIndex + jump : itemIndex - jump;

  if (newIndex < groupSize && newIndex >= 0) {
    return {
      item: groups[groupIndex][newIndex],
      coords: [groupIndex, newIndex]
    };
  }

  var nextGroupIndex = getNextGroup(groupIndex, groups, newIndex >= groupSize ? 1 : -1);

  if (nextGroupIndex == null) {
    return emptyValue;
  }

  var nextGroup = groups[nextGroupIndex];
  var nextItemIndex = direction === 1 ? 0 : nextGroup.length - 1;
  var item = nextGroup[nextItemIndex];

  if (item == null) {
    return emptyValue;
  }

  return {
    item,
    coords: [nextGroupIndex, nextItemIndex]
  };
}

function toGroups(namedGroups) {
  return namedGroups.map(r => r.emojis);
}

function normalizeCounter(counter, groups) {
  var totalSize = groups.reduce((prev, cur) => prev + cur.length, 0);

  if (totalSize === 0) {
    return;
  }

  while (counter < 0) {
    counter += totalSize;
  }

  counter = counter % totalSize;
  return counter;
}